/***************************************************************************
 *   Copyright (C) 2008 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "GuitoneCore.h"
#include "Preferences.h"
#include "Settings.h"

#ifdef Q_WS_MACX
#include "CocoaUtil.h"
#endif

#include <QMessageBox>
#include <QTimer>

GuitoneCore::GuitoneCore(int & argc, char ** argv)
  : QApplication(argc, argv), monotoneManager(0)
{
#ifndef Q_WS_MACX
    updateDialog = 0;
#else
    CocoaUtil::initialize();
#endif

    // check for updates immediatly on launch
    if (Settings::getBool("CheckForUpdates", true))
    {
        checkForApplicationUpdates(true);
    }

    monotoneManager = new MonotoneManager();

    connect(
        monotoneManager, SIGNAL(threadAborted(const QString &)),
        this, SLOT(threadAborted(const QString &))
    );

    QTimer::singleShot(0, this, SLOT(setMonotoneBinaryPath()));
}

GuitoneCore::~GuitoneCore()
{
    delete monotoneManager;
#ifndef Q_WS_MACX
    if (updateDialog) delete updateDialog;
#endif
}

void GuitoneCore::setMonotoneBinaryPath()
{
    if (monotoneManager->monotoneBinaryPathSet())
    {
        W("monotone binary path already set");
    }

    QString installedVersion;
    if (!monotoneManager->setMonotoneBinaryPath(
            Settings::getMtnBinaryPath(),
            Settings::getBool("RelaxedVersionCheck", false),
            installedVersion))
    {
        QMessageBox::critical(
            NULL,
            tr("Error"),
            tr("The path to the monotone binary is either invalid "
               "or points to a version of monotone with which guitone can't "
               "work with. Guitone requires monotone with an interface version "
               "between '%1' and '%2'. Your installation has the interface "
               "version '%3'.")
                   .arg(MonotoneManager::MinInterfaceVersion)
                   .arg(MonotoneManager::MaxInterfaceVersion)
                   .arg(installedVersion),
            QMessageBox::Ok, 0, 0
        );

        // note: the pref dialog tries to set the mtn binary path on its
        // own again and can only be accepted successfully, if the user
        // enters/selects a valid monotone binary path
        Preferences dialog(NULL);

        // the user has rejected the dialog, i.e. made no new settings
        if (dialog.exec() == QDialog::Rejected)
        {
            return;
        }
    }
    D("binary path was set");
    emit monotoneBinaryPathSet();
}

void GuitoneCore::quit()
{
    Settings::sync();
    QApplication::quit();
}

void GuitoneCore::checkForApplicationUpdates(bool silent)
{
#ifdef Q_WS_MACX
    CocoaUtil::checkForUpdates(silent);
#else
    if (!updateDialog)
    {
        updateDialog = new ApplicationUpdate(NULL);
    }
    updateDialog->checkForUpdates(silent);
#endif
}

void GuitoneCore::threadAborted(const QString & message)
{
    QMessageBox::critical(
        NULL,
        tr("Error"),
        message,
        QMessageBox::Ok, 0, 0
    );
}

