/***************************************************************************
 *   Copyright (C) 2006 by Jean-Louis Fuchs                                 *
 *   ganwell@fangorn.ch                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "GetBranchLog.h"

GetBranchLog::GetBranchLog(QObject * parent, const DatabaseFile & db, const QString & b)
    : QObject(parent), branch(b), databaseFile(db)
{
    selectModel   = new Select(this, databaseFile);
    revSortModel  = new Toposort(this, databaseFile);
    ancestorModel = new Ancestors(this, databaseFile);
    revSortModel->setSourceModel(ancestorModel);

    connect(
        selectModel, SIGNAL(selectionRead()),
        this, SLOT(selectionReady())
    );

    connect(
        revSortModel, SIGNAL(sortingFinished()),
        this, SLOT(revisionsReady())
    );

    selectModel->readSelection("h:" + branch);
}

GetBranchLog::~GetBranchLog()
{
    delete selectModel;
    delete revSortModel;
    delete ancestorModel;
}

void GetBranchLog::selectionReady()
{
    QStringList heads;
    int count = selectModel->rowCount(QModelIndex());

    for (int i = 0; i < count; i++)
    {
        heads.append(
            selectModel->data(
                selectModel->index(i, 0, QModelIndex()),
                Qt::DisplayRole
            ).toString()
        );
    }
    revisions += heads;
    ancestorModel->readAncestors(heads);
}

void GetBranchLog::revisionsReady()
{
    int count = revSortModel->rowCount(QModelIndex());
    for (int i = 0; i < count; i++)
    {
        revSortModel->sort(0, Qt::DescendingOrder);
        revisions.append(
            revSortModel->QSortFilterProxyModel::data(
                revSortModel->index(
                    i, 0,
                    QModelIndex()
                ),
                Qt::DisplayRole
            ).toString()
        );
    }
    emit branchLogRead(branch, revisions);
}

