/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef INVENTORY_MODEL_H
#define INVENTORY_MODEL_H

#include "Inventory.h"

#include <QAbstractItemModel>
#include <QStack>

struct NodeInsertion {
    NodeInsertion(ModelItem * p, int s, int e)
      : parent(p), start(s), end(e) {}
    ModelItem * parent;
    int start;
    int end;
};

class InventoryModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    InventoryModel(QObject *);
    ~InventoryModel();

    QVariant data(const QModelIndex &, int) const;
    Qt::ItemFlags flags(const QModelIndex &) const;
    QVariant headerData(int, Qt::Orientation, int) const;
    QModelIndex index(int, int, const QModelIndex &) const;
    QModelIndex parent(const QModelIndex &) const;
    int rowCount(const QModelIndex &) const;
    int columnCount(const QModelIndex &) const;
    bool canFetchMore( const QModelIndex &) const;
    void fetchMore(const QModelIndex &);

public slots:
    //! forwards
    void setWorkspacePath(const WorkspacePath &);
    void refresh(const QString & path = QString());

private:
    QModelIndex indexFromItem(ModelItem *, int) const;

    Inventory * inventory;
    WorkspacePath workspacePath;
    QStack<NodeInsertion *> nodeInsertions;

private slots:
    void triggerBeginInsertRows(ModelItem *, int, int);
    void triggerEndInsertRows();

    void triggerBeginRemoveRows(ModelItem *, int, int);
    void triggerEndRemoveRows();

    void triggerDataChanged(ModelItem *);

    void maybeResetModel(const QString &);

signals:
    //! forward
    void invalidWorkspaceFormat(const QString &);
    void pathsInserted(const QStringList &);
    void pathsRemoved(const QStringList &);
};

#endif

