/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef INVENTORY_VIEW_H
#define INVENTORY_VIEW_H

#include "TreeView.h"
#include "InventoryViewDelegate.h"
#include "InventoryItem.h"

#include <QAction>
#include <QSortFilterProxyModel>

class InventoryItem;

class InventoryView : public TreeView
{
    Q_OBJECT

public:
    enum Type { FolderTree, FileList };

    InventoryView(QWidget *, const QString &);
    ~InventoryView();

    void setModel(QSortFilterProxyModel *);
    void setType(Type);

signals:
    void directoryChanged(const QModelIndex &);
    void openFile(const QString &);
    void diffRevision(const QString &, const QString &, const QString &);
    void diffFile(const QString &);
    void fileHistory(const QString &);
    void commitRevision(const QStringList &);
    void refreshPath(const QString &);

private:
    enum DefaultAction { None, Chdir, Open, FileDiff, Commit };

    void setModel(QAbstractItemModel *);
    QSet<const ModelItem *> getSelectedItems() const;
    DefaultAction getDefaultAction(const QModelIndex &) const;
    void createAndConnectContextActions();
    void closeEvent();

    inline QModelIndex sourceIndex(const QModelIndex &, bool useIndexModel = false) const;
    inline QModelIndex proxyIndex(const QModelIndex &) const;
    static inline const ModelItem * modelItem(const QModelIndex &);

    QAction * actChdir;
    QAction * actOpen;
    QAction * actAdd;
    QAction * actRemove;
    QAction * actCommit;
    QAction * actIgnore;
    QAction * actUnignore;
    QAction * actRevert;
    QAction * actRename;
    QAction * actFileDiff;
    QAction * actFileHistory;
    QAction * actRevisionDiff;
    QAction * actRefresh;

    QAction * actAddMultiple;
    QAction * actRemoveMultiple;
    QAction * actCommitMultiple;
    QAction * actIgnoreMultiple;
    QAction * actUnignoreMultiple;
    QAction * actRevertMultiple;

    Type type;

    InventoryViewDelegate invViewDelegate;

private slots:
    void expandRootNode(const QModelIndex &, const QModelIndex &);
    void changeDirectory(const QModelIndex &);
    void slotContextMenuRequested(const QModelIndexList &, const QPoint &);
    void itemClicked(const QModelIndex & index);
    void itemDoubleClicked(const QModelIndex & index);
    void setBackgroundImage();

    void slotChdir();
    void slotOpen();
    void slotAdd();
    void slotRemove();
    void slotCommit();
    void slotIgnore();
    void slotUnignore();
    void slotRevert();
    void slotRename();
    void slotFileDiff();
    void slotFileHistory();
    void slotRevisionDiff();
    void slotRefresh();
};

#endif
