/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "DialogManager.h"
#include "vocab.h"

DialogManager::DialogManager(QWidget * parentWidget)
  : QObject(parentWidget), about(0), preferences(0)
{}

DialogManager::~DialogManager()
{
    if (about)          delete about;
    if (preferences)    delete preferences;
}

QWidget * DialogManager::parentWidget() const
{
    return reinterpret_cast<QWidget *>(parent());
}

void DialogManager::closeAllDialogs()
{
    if (about)          about->close();
    if (preferences)    preferences->close();
    emit allDialogsClosed();
}

void DialogManager::showAbout()
{
    if (!about)
    {
        about = new About(parentWidget());
    }
    showDialog(about);
}

void DialogManager::showPreferences()
{
    if (!preferences)
    {
        preferences = new Preferences(parentWidget());
    }
    showDialog(preferences);
}

void DialogManager::showDialog(Dialog * dlg)
{
    if (!openDialogs.contains(dlg))
    {
        connect(
            dlg, SIGNAL(finished(int)),
            this, SLOT(dialogFinished())
        );
        openDialogs.push(dlg);
    }

    dlg->raise();
    dlg->activateWindow();
    dlg->show();
}

void DialogManager::dialogFinished()
{
    Dialog * dlg = qobject_cast<Dialog *>(sender());
    I(openDialogs.contains(dlg));

    disconnect(
        dlg, SIGNAL(finished(int)),
        this, SLOT(dialogFinished())
    );

    // this is the simple case, the closed dialog is on top of the stack
    if (openDialogs.top() == dlg)
    {
        openDialogs.pop();
    }
    else
    {
        // Now apparently the user accepted / rejected a window from which he
        // navigated to other windows. Since all of these are not modal, but
        // show dependent information, we close them one after another.
        // Since this triggers this slot again, we need to ensure before
        // that this code path is aborted.
        // FIXME: not sure this is really what users want, but we're cleaning
        // up here for them... in case a user works really in parallel, this
        // could have weird side effects, like closing dialogs which are not
        // supposed to be closed, just because they're created after the main
        // dialog...
        while (true)
        {
            Dialog * other = openDialogs.pop();
            disconnect(
                other, SIGNAL(finished(int)),
                this, SLOT(dialogFinished())
            );
            other->reject();
            if (other == dlg) break;
        }
    }

    // is there anything to raise?
    if (!openDialogs.isEmpty())
    {
        Dialog * other = openDialogs.top();
        other->raise();
        other->activateWindow();
        return;
    }

    emit allDialogsClosed();
}

