/***************************************************************************
 *   Copyright (C) 2006 by Ingo Maindorfer                                 *
 *   ingo@liquidcooling.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "GenerateKeypair.h"
#include "MonotoneUtil.h"

#include <QMessageBox>
#include <QStringList>

GenerateKeypair::GenerateKeypair(QWidget * parent, const DatabaseFile & db)
  : Dialog(parent), databaseFile(db)
{
    setupUi(this);
    Dialog::init();
}

GenerateKeypair::~GenerateKeypair() {}

void GenerateKeypair::accept()
{
    if (lineKeyName->text().size() == 0 ||
        lineKeyPasswd->text().size() == 0 ||
        lineKeyPasswd2->text().size() == 0)
    {
        QMessageBox::information(
            this,
            tr("Missing information"),
            tr("Please fill out all fields."),
            QMessageBox::Ok
        );
        return;
    }

    if (lineKeyPasswd->text().compare(lineKeyPasswd2->text()) != 0)
    {
        QMessageBox::information(
            this,
            tr("Password mismatch"),
            tr("The entered passwords did not match."),
            QMessageBox::Ok
        );
        return;
    }

    MonotoneTask out = MonotoneUtil::runSynchronousDatabaseTask(
        databaseFile,
        MonotoneTask(QStringList() << "genkey"
                    << lineKeyName->text()
                    << lineKeyPasswd->text()
        )
    );
    if (!out.isFinished()) F("task aborted");

    if (out.getReturnCode() > 0)
    {
        QMessageBox::critical(
             this,
             tr("Error creating keypair"),
             tr("There was an error creating the keypair:\n%1")
                .arg(out.getDecodedOutput()),
             QMessageBox::Ok, 0, 0
        );
        return;
    }

    emit keypairGenerated();
    done(QDialog::Accepted);
}

